/* the cloc version 2 with the new param */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "cloc.h"



static void pp(const char *label, const byte *c)
{
#ifdef DBG
    printf("%s: ", label);
    int i;
    for (i = 0; i < 16; i++) {
        printf("%02X", c[i]);
    }

    printf("\n");
#endif    
}


/* setup key */
int ae_init(ae_cxt* cxt, const byte* userkey, int keylen)
{
	if(keylen != STATE_LEN){
		return KEY_ERROR;
	}
	cxt->userkey = (byte*)userkey;
	cxt->pt_ekey = &(cxt->ekey);
	AES_set_encrypt_key(userkey, keylen*8, cxt->pt_ekey); /* set the encryption key */
	cxt->klen = keylen;
    
	return SUCCESS;
}

/* associated data and nonce length in byte
 * nlen should be less than 16 bytes and larger than 0 byte;
 */
int process_ad(ae_cxt* cxt, const byte* ad, unsigned long long adlen, const byte* nonce, unsigned long long nlen)
{
	byte* state = cxt->es;
	AES_KEY *ekey = cxt->pt_ekey;

	cxt->ad = (byte*) ad;
	cxt->adlen = adlen;
	cxt->nonce = (byte*) nonce;
	cxt->nlen = nlen;
	/* process the first block */
	int ozp = 0;
	if(adlen < STATE_LEN){       // less than one block
		memcpy(state, ad, adlen);
		memset(state+adlen, 0, STATE_LEN-adlen);
		state[adlen] = 0x80;
		ozp = 1;                // one-zero padding works only if the adlen < 16
	}
	else{                       // full first block
		memcpy(state, ad, STATE_LEN);
	}

	/* apply fix0 and the E_k */
	int fix0 = state[0] & 0x80;    /* test if the MSB is zero */
	state[0] &= 0x7f;

	/* apply the first encryption */
    pp("(A ) AES In : ", state);
	AES_encrypt(state, state, ekey);
    pp("(A ) AES Out: ", state);
	/* when fix0 works, apply h */
	if(fix0){
		u32* wd = (u32*) state;
        pstate2("h: ", state, STATE_LEN);
		h(wd[0], wd[1], wd[2], wd[3]);
        pstate2("", state, STATE_LEN);
        pp("(fix0) State: ", state);
	}
	else{
        // do nothing
	}

	/* process the middle normal blocks of ad */
	unsigned long long i;
	for(i = 1; i < (unsigned long long)adlen/STATE_LEN; i++){
		xor_block((block*)state, (block*)ad+i);
        pp("(AX) AES In : ", state);
		AES_encrypt(state, state, ekey);
        pp("(AX) AES Out: ", state);
	}
	/* process the last block partial block if any */
	unsigned long long lastblocklen = adlen % STATE_LEN;
	if((adlen > STATE_LEN) && lastblocklen){
		xor_bytes(state, ad+i*STATE_LEN, lastblocklen);
		state[lastblocklen] ^= 0x80;
        pp("(AF) AES In : ", state);
		AES_encrypt(state, state, ekey);
        pp("(AF) AES Out: ", state);
		ozp = 1;
	}

	/* process the nonce 
	 * 1. first byte is: PARAM
	 * 2. then the nonce value
	 * 3. padding if any (at the moment, the parameter set contains padding for all choices)
	 */
    pp("(AF) State  : ", state);
	state[0] ^= PARAM;
	xor_bytes(state+1, nonce, nlen);

	/* apply padding to nonce */
	if((nlen+1) != STATE_LEN)
		state[nlen+1] ^= 0x80;

    pp("(AF) Nonce  : ", state);
    
	/* apply f1 or f2 to get V */
	if(ozp){ // apply f2
		u32* wd = (u32*) state;
        pstate2("f2: ", state, STATE_LEN);
		f2(wd[0], wd[1], wd[2], wd[3]);
        pstate2("", state, STATE_LEN);
	}
	else{ // apply f1jj
		u32* wd = (u32*) state;
        pstate2("f1: ", state, STATE_LEN);
		f1(wd[0], wd[1], wd[2], wd[3]);
        pstate2("", state, STATE_LEN);
	}
    pp("(AF) OZP    : ", state);

	memcpy(cxt->ts, state, STATE_LEN);
    pp("(AF) AES In : ", state);
	AES_encrypt(state, state, ekey);
    pp("(AF) AES Out: ", state);

	return SUCCESS;
}

/*
 * plaintext, message length in byte, ciphertext, associated data, and associated data length in byte tag, and tag length in byte
 */
int ae_encrypt(ae_cxt* cxt, byte* pt, unsigned long long mlen, byte* ct, byte* tag, unsigned long long tlen, int enc_dec)
{
	cxt->pt = pt;
	cxt->ptlen = mlen;
	cxt->ct = ct;
	cxt->ctlen = mlen;
	cxt->tag = tag;
	cxt->tlen = tlen;

	byte* es = cxt->es;
	byte* ts = cxt->ts;
	AES_KEY *ekey = cxt->pt_ekey;
    
    u32* wd = (u32*) ts;
    if(mlen != 0){
        pstate2("g2: ", ts, STATE_LEN);
    	g2(wd[0], wd[1], wd[2], wd[3]);
        pstate2("", ts, STATE_LEN);
        pp("(M2) AES In : ", cxt->ts);
        AES_encrypt(cxt->ts, cxt->ts, ekey);
        pp("(M2) AES Out: ", cxt->ts);        
    }
    else{
        pstate2("g1: ", ts, STATE_LEN);
        g1(wd[0], wd[1], wd[2], wd[3]);
        pstate2("", ts, STATE_LEN);
        pp("(M2) AES In : ", cxt->ts);
        AES_encrypt(cxt->ts, cxt->ts, ekey);
        pp("(M2) AES Out: ", cxt->ts);
        memcpy(tag, ts, tlen);
        return SUCCESS;
    }
    

    printf(" == Full block(s) \n");
    pp("sm : ", es);
    pp("st : ", ts);
    
	unsigned long long pc = 0;
	while((pc + STATE_LEN) < mlen){
        if(enc_dec == ENC){ // encryption
            xor_block((block*)es, (block*)((byte*)(pt+pc)));
            memcpy(ct+pc, es, STATE_LEN);
        }
        else{ // decryption
            xor_bytes2(pt+pc, ct+pc, es, STATE_LEN);
            memcpy(es, ct+pc, STATE_LEN);
        }
		xor_block((block*)ts, (block*)es);
        pp("(M2) AES In : ", ts);
		AES_encrypt(ts, ts, ekey);
        pp("(M2) AES Out: ", ts);
		/* apply fix1 */
		es[0] |= 0x80;
        pp("(M1) AES In : ", es);
		AES_encrypt(es, es, ekey);
        pp("(M1) AES Out: ", es);
		pc += STATE_LEN;
	}

    printf(" == Last block \n");
    pp("(T ) StateSm: ", es);
    
	/* process the last block */
	unsigned long long lastblocklen = mlen - pc;
    if(enc_dec == ENC){ // encryption
        xor_bytes(es, pt+pc, lastblocklen);
        memcpy(ct+pc, es, lastblocklen);
    }
    else{ // decryption
        xor_bytes2(pt+pc, ct+pc, es, lastblocklen);
        memcpy(es, ct+pc, lastblocklen);
    }
    pp("(T ) StateSm: ", es);
    pp("(T ) StateSt: ", ts);
    
	xor_bytes(ts, es, lastblocklen);
    pp("(T ) StateSt: ", ts);
	if(lastblocklen != STATE_LEN){ // apply f2
        /* apply padding only when last message block is not full */
        ts[lastblocklen] ^= 0x80;
		u32* wd = (u32*) ts;
        pstate2("f2: ", ts, STATE_LEN);
		f2(wd[0], wd[1], wd[2], wd[3]);
        pstate2("", ts, STATE_LEN);
        pp("(F2) State  : ", ts);
	}
	else{ // apply f1
		u32* wd = (u32*) ts;
        pstate2("f1: ", ts, STATE_LEN);
		f1(wd[0], wd[1], wd[2], wd[3]);
        pstate2("", ts, STATE_LEN);
        pp("(F1) State  : ", ts);
	}

    #ifdef DBG
    printf(" == Tag \n" );
    #endif
    pp("(M2) AES In : ", ts);
	AES_encrypt(ts, ts, ekey);
    pp("(M2) AES Out: ", ts);
	memcpy(tag, ts, tlen);

	return SUCCESS;
}

