// wbob_pi64.c
// 10-Sep-15  Markku-Juhani O. Saarinen <mjos@iki.fi>
// A version with large tables.

#include "stribob.h"
#include "wbob_tab64.h"

#include <stdio.h>

#define DBG_ROUND 0
#define OPTIMIZE 1

#if (__BYTE_ORDER__ != __ORDER_LITTLE_ENDIAN__)
// Big Endian
#define WBOB_SM(i, x) wbob_sm##i [((x) >> (56 - 8 * i)) & 0xFF]
#else
// Little Endian
#define WBOB_SM(i, x) wbob_sm##i [((x) >> (8 * i)) & 0xFF]
#endif

uint8_t wbob_sbox[256] = {
    0x18, 0x23, 0xC6, 0xE8, 0x87, 0xB8, 0x01, 0x4F, 0x36, 0xA6, 0xD2, 0xF5, 0x79, 0x6F, 0x91, 0x52,
    0x60, 0xBC, 0x9B, 0x8E, 0xA3, 0x0C, 0x7B, 0x35, 0x1D, 0xE0, 0xD7, 0xC2, 0x2E, 0x4B, 0xFE, 0x57,
    0x15, 0x77, 0x37, 0xE5, 0x9F, 0xF0, 0x4A, 0xDA, 0x58, 0xC9, 0x29, 0x0A, 0xB1, 0xA0, 0x6B, 0x85,
    0xBD, 0x5D, 0x10, 0xF4, 0xCB, 0x3E, 0x05, 0x67, 0xE4, 0x27, 0x41, 0x8B, 0xA7, 0x7D, 0x95, 0xD8,
    0xFB, 0xEE, 0x7C, 0x66, 0xDD, 0x17, 0x47, 0x9E, 0xCA, 0x2D, 0xBF, 0x07, 0xAD, 0x5A, 0x83, 0x33,
    0x63, 0x02, 0xAA, 0x71, 0xC8, 0x19, 0x49, 0xD9, 0xF2, 0xE3, 0x5B, 0x88, 0x9A, 0x26, 0x32, 0xB0,
    0xE9, 0x0F, 0xD5, 0x80, 0xBE, 0xCD, 0x34, 0x48, 0xFF, 0x7A, 0x90, 0x5F, 0x20, 0x68, 0x1A, 0xAE,
    0xB4, 0x54, 0x93, 0x22, 0x64, 0xF1, 0x73, 0x12, 0x40, 0x08, 0xC3, 0xEC, 0xDB, 0xA1, 0x8D, 0x3D,
    0x97, 0x00, 0xCF, 0x2B, 0x76, 0x82, 0xD6, 0x1B, 0xB5, 0xAF, 0x6A, 0x50, 0x45, 0xF3, 0x30, 0xEF,
    0x3F, 0x55, 0xA2, 0xEA, 0x65, 0xBA, 0x2F, 0xC0, 0xDE, 0x1C, 0xFD, 0x4D, 0x92, 0x75, 0x06, 0x8A,
    0xB2, 0xE6, 0x0E, 0x1F, 0x62, 0xD4, 0xA8, 0x96, 0xF9, 0xC5, 0x25, 0x59, 0x84, 0x72, 0x39, 0x4C,
    0x5E, 0x78, 0x38, 0x8C, 0xD1, 0xA5, 0xE2, 0x61, 0xB3, 0x21, 0x9C, 0x1E, 0x43, 0xC7, 0xFC, 0x04,
    0x51, 0x99, 0x6D, 0x0D, 0xFA, 0xDF, 0x7E, 0x24, 0x3B, 0xAB, 0xCE, 0x11, 0x8F, 0x4E, 0xB7, 0xEB,
    0x3C, 0x81, 0x94, 0xF7, 0xB9, 0x13, 0x2C, 0xD3, 0xE7, 0x6E, 0xC4, 0x03, 0x56, 0x44, 0x7F, 0xA9,
    0x2A, 0xBB, 0xC1, 0x53, 0xDC, 0x0B, 0x9D, 0x6C, 0x31, 0x74, 0xF6, 0x46, 0xAC, 0x89, 0x14, 0xE1,
    0x16, 0x3A, 0x69, 0x09, 0x70, 0xB6, 0xD0, 0xED, 0xCC, 0x42, 0x98, 0xA4, 0x28, 0x5C, 0xF8, 0x86};
    
void printState(uint8_t *s) 
{
    int j;
    for (j = 0; j<64; j++) {
        printf("%02X",s[j]);
        if (((j+1) % 4)  == 0) printf(" ");
        if (((j+1) % 16) == 0) printf("\n");
        
    }
    printf("\n");
}    


#ifndef OPTIMIZE

void wbob_pi(w512_t *s512)
{
    uint8_t *st = &s512->b[0];

    int r, i, j; // loop variables
    uint8_t t[64], sb[64], x, *pt, *rc; // work variables
    
#if DBG
    uint8_t tt[] = {0x77, 0x38, 0xe1, 0xb5, 0x41, 0xa0, 0x36, 0xea,
                    0x45, 0x8d, 0x50, 0xf8, 0x0f, 0xa0, 0x1c, 0x44,
                    0x72, 0x88, 0xce, 0x97, 0xd1, 0xa0, 0xdc, 0xf0,
                    0x16, 0x95, 0xff, 0xd6, 0xe7, 0x1d, 0x09, 0x25,
                    0x33, 0xbe, 0x30, 0x9f, 0x01, 0x2a, 0x59, 0x09,
                    0x72, 0x91, 0x14, 0x59, 0x5f, 0x08, 0x6e, 0x76,
                    0x07, 0x18, 0xaf, 0xe3, 0x65, 0xbc, 0x09, 0xde,
                    0xb6, 0xaf, 0xa1, 0x80, 0xbc, 0xec, 0x2a, 0x98};
    for (i=0; i<64; i++) {
        s512->b[i] = tt[i];
    }
    printf("BlockIn:\n");
    printState(s512);
#endif
    
    for (r = 0; r < 12; r++) { // 12 rounds
        for (i = 0; i < 64; i++) {
            sb[i] = wbob_sbox[st[i]];
        }

        #if DBG_ROUND
            printf("\tAfter SubByte:\n", r+1);
            printState(sb);
            
            // for (i = 0; i < 64; i++)
                // printf("%d -> %d\n", i, (i & 7) + ((i + (i << 3)) & 070));
        #endif
        
        for (i = 0; i < 64; i++) {
            t[(i & 7) + ((i + (i << 3)) & 070)] = sb[i]; // P Permutation..            
        }
        
        #if DBG_ROUND
            printf("\tAfter Permute:\n", r+1);
            printState(t);
        #endif

        // Round constants C come from the S-box
        rc = (uint8_t *) &wbob_sbox[8 * r];
        for (i = 0; i < 8; i++)
            st[i] = rc[i]; // C in first 8
        for (i = 8; i < 64; i++)
            st[i] = 0; // zero the rest

        // Apply the circular, low weight MDS matrix
        for (i = 0; i < 64; i += 8) {
            pt = &st[i]; // start of row
            for (j = 0; j < 8; j++) {
                x = t[i + j]; // Circular MDS
                pt[j & 7] ^= x; // 01
                pt[(j + 1) & 7] ^= x; // 01
                pt[(j + 3) & 7] ^= x; // 01
                pt[(j + 5) & 7] ^= x; // odd
                pt[(j + 7) & 7] ^= x; // odd
                x = (x << 1) ^ (x & 0x80 ? 0x1D : 0x00); // double x
                pt[(j + 6) & 7] ^= x; // 02
                x = (x << 1) ^ (x & 0x80 ? 0x1D : 0x00); // redouble to 4x
                pt[(j + 2) & 7] ^= x; // 04
                pt[(j + 5) & 7] ^= x; // 01 + 04 = 05
                x = (x << 1) ^ (x & 0x80 ? 0x1D : 0x00); // redouble to 8x
                pt[(j + 4) & 7] ^= x; // 08
                pt[(j + 7) & 7] ^= x; // 01 + 08 = 09
            }
        }
        
        
        // // Round constants C come from the S-box
        // rc = (uint8_t *) &wbob_sbox[8 * r];
        // for (i = 0; i < 8; i++)
            // pt[i] ^= rc[i]; // C in first 8
        
        #if DBG_ROUND
            printf("\tRound %d:\n", r+1);
            printState(s512);            
            exit(0);
        #endif
    }
}
#else

void wbob_pi(w512_t *s512)
{
#if DBG
    int j;
    printf("BlockIn:\n");
    for (j = 0; j<64; j++) {
        printf("%02X",s512->b[j]);
        if (((j+1) % 4)  == 0) printf(" ");
        if (((j+1) % 16) == 0) printf("\n");
    }
    printf("\n");
#endif

	int i;
	uint64_t k[8], l[8];

	// load block
	for (i = 0; i < 8; i++)
		k[i] = s512->q[i];

	for (i = 0; i < 12; i++) {

		l[0] =	WBOB_SM(0, k[0]) ^	WBOB_SM(1, k[7]) ^
				WBOB_SM(2, k[6]) ^	WBOB_SM(3, k[5]) ^
				WBOB_SM(4, k[4]) ^	WBOB_SM(5, k[3]) ^
				WBOB_SM(6, k[2]) ^	WBOB_SM(7, k[1]) ^ wbob_rc[i];

		l[1] =	WBOB_SM(0, k[1]) ^	WBOB_SM(1, k[0]) ^
				WBOB_SM(2, k[7]) ^	WBOB_SM(3, k[6]) ^
				WBOB_SM(4, k[5]) ^	WBOB_SM(5, k[4]) ^
				WBOB_SM(6, k[3]) ^	WBOB_SM(7, k[2]);

		l[2] =	WBOB_SM(0, k[2]) ^	WBOB_SM(1, k[1]) ^
				WBOB_SM(2, k[0]) ^	WBOB_SM(3, k[7]) ^
				WBOB_SM(4, k[6]) ^	WBOB_SM(5, k[5]) ^
				WBOB_SM(6, k[4]) ^	WBOB_SM(7, k[3]);

		l[3] =	WBOB_SM(0, k[3]) ^	WBOB_SM(1, k[2]) ^
				WBOB_SM(2, k[1]) ^	WBOB_SM(3, k[0]) ^
				WBOB_SM(4, k[7]) ^	WBOB_SM(5, k[6]) ^
				WBOB_SM(6, k[5]) ^	WBOB_SM(7, k[4]);

		l[4] =	WBOB_SM(0, k[4]) ^	WBOB_SM(1, k[3]) ^
				WBOB_SM(2, k[2]) ^	WBOB_SM(3, k[1]) ^
				WBOB_SM(4, k[0]) ^	WBOB_SM(5, k[7]) ^
				WBOB_SM(6, k[6]) ^	WBOB_SM(7, k[5]);

		l[5] =	WBOB_SM(0, k[5]) ^	WBOB_SM(1, k[4]) ^
				WBOB_SM(2, k[3]) ^	WBOB_SM(3, k[2]) ^
				WBOB_SM(4, k[1]) ^	WBOB_SM(5, k[0]) ^
				WBOB_SM(6, k[7]) ^	WBOB_SM(7, k[6]);

		l[6] =	WBOB_SM(0, k[6]) ^	WBOB_SM(1, k[5]) ^
				WBOB_SM(2, k[4]) ^	WBOB_SM(3, k[3]) ^
				WBOB_SM(4, k[2]) ^	WBOB_SM(5, k[1]) ^
				WBOB_SM(6, k[0]) ^	WBOB_SM(7, k[7]);

		l[7] =	WBOB_SM(0, k[7]) ^	WBOB_SM(1, k[6]) ^
				WBOB_SM(2, k[5]) ^	WBOB_SM(3, k[4]) ^
				WBOB_SM(4, k[3]) ^	WBOB_SM(5, k[2]) ^
				WBOB_SM(6, k[1]) ^	WBOB_SM(7, k[0]);

        k[0] = l[0];
        k[1] = l[1];
        k[2] = l[2];
        k[3] = l[3];
        k[4] = l[4];
        k[5] = l[5];
        k[6] = l[6];
        k[7] = l[7];
        #if DBG_ROUND
            printf("\tRound %d:\n", i+1);
            printf("\t%08X %08X %08X %08X\n", (uint32_t) (l[7] >> 32), (uint32_t) l[7], (uint32_t) (l[6] >> 32), (uint32_t) l[6]);
            printf("\t%08X %08X %08X %08X\n", (uint32_t) (l[5] >> 32), (uint32_t) l[5], (uint32_t) (l[4] >> 32), (uint32_t) l[4]);
            printf("\t%08X %08X %08X %08X\n", (uint32_t) (l[3] >> 32), (uint32_t) l[3], (uint32_t) (l[2] >> 32), (uint32_t) l[2]);
            printf("\t%08X %08X %08X %08X\n", (uint32_t) (l[1] >> 32), (uint32_t) l[1], (uint32_t) (l[0] >> 32), (uint32_t) l[0]);
            printf("\n");
        #endif
	}

	for (i = 0; i < 8; i++)
		s512->q[i] = k[i];

#if DBG
    printf("\tBlockOut:\n");
    printState(s512);    
#endif
}

#endif

