#!/usr/bin/env python3
# -*- coding: utf-8 -*

##========================================================================
## DO NOT MODIFY THIS PART
##========================================================================
import os
import sys


try:
    from aeadtvgen import aeadtvgen
except ImportError:
    print("================================================================")
    print("Warning: aeadtvgen not installed!!!! Use source package instead.")
    print("================================================================")    
    sys.path.insert(0,os.path.abspath('..'))    
    from aeadtvgen.cli import aeadtvgen

if __name__ == '__main__':
##========================================================================
## BASIC ARGUMENTS
## ENTER PARAMETERS OF YOUR SPECIFIC CIPHER AND ITS IMPLEMENTATION HERE
##========================================================================
    # Create the list of arguments for aeadtvgen
    basic_args = [
        os.path.realpath('../../lib'),      ## Library path
        'aes128n12t8clocv2',                ## Library name
        '--io', '32', '32',                 ## I/O width: PDI/DO and SDI width, respectively.
        '--key_size', '128',                ## Key size
        '--npub_size', '96',                ## Npub size
        '--nsec_size', '0',                 ## Nsec size
        '--tag_size', '64',                 ## Tag size
        '--block_size',    '128',           ## Data block size
        '--block_size_ad', '128',           ## AD block size
        # '--ciph_exp',                     ## Ciphertext expansion
        # '--add_partial',                  ## ciph_exp option: add partial bit
        # '--ciph_exp_noext',               ## ciph_exp option: no block extension when a message is a multiple of a block size
        # '--offline',                      ## Offline cipher (Adds Length segment as the first input segment)
        '--dest', '../KAT/aes128n12t8clocv2',          ## destination folder
        '--max_ad', '200',                  ## Maximum random AD size
        '--max_d', '200',                   ## Maximum random message size
        '--max_io_per_line', '8',           ## Max number of w-bit I/O word per line
        '--human_readable',                 ## Generate a human readable text file
        '--verify_lib',                     ## Verify reference enc/dec in reference code
                                            ## Note: (This option performs decryption check for
                                            ##        each encryption operation used to
                                            ##        create the test vector)
        '--msg_format', 'npub', 'ad', 'data', 'tag'                 ## Message format
    ]

##=========================================================================
## VERIFICATION PHASES & CORRESPONDING TEST VECTOR GENERATION OPTIONS
## ADJUST TO MEET THE REQUIREMENTS OF YOUR IMPLEMENTATION AND TESTING STRATEGY
##=========================================================================
## PHASE 1 - SINGLE AD & MESSAGE/CIPHERTEXT BLOCK
##=========================================================================
    gen_single = ['--gen_single',
        '0',                                # Encrypt(0)/decrypt(1)
        '000102030405060708090A0B0C0D0E0F', #Key
        '000102030405060708090A0B0C0D0E0F', #Npub
        '000102030405060708090A0B0C0D0E0F', #Nsec (Ignored: nsec_size=0)
        '000102030405060708090A0B0C0D0E0F', #AD
        '000102030405060708090A0B0C0D0E0F', #DATA
        ]
##=========================================================================
## PHASE 2 - RANDOM INPUTS WITH CUSTOM SELECTED SIZES
##=========================================================================
    gen_custom = ['--gen_custom',   # Note: Use this format!!
        '''\
        True,   False,      0,          20:
        0,      1,          100,        500
        ''']
##=========================================================================
## PHASE 3 - EMPTY MESSAGE, EMPTY AD, BASIC MESSAGE/AD SIZES
##=========================================================================
    gen_test_routine = '--gen_test_routine 1 20 2'.split()
##=========================================================================
## PHASE 4 - RANDOMLY GENERATED TEST VECTORS WITH VARYING AD, MESSAGE,
##           & CIPHERTEXT LENGTHS
##=========================================================================
    gen_random = '--gen_random 10'.split()
##=========================================================================


##========================================================================
## CHOOSE YOUR PHASE OF THE VERIFICATION PROCESS HERE
## BY UNCOMMENTING THE RESPECTIVE LINE
##========================================================================
## PHASE 1:
#    args = basic_args + gen_single
##========================================================================
## PHASE 2:
#    args = basic_args + gen_custom
##========================================================================
## PHASE 3:
#    args = basic_args + gen_test_routine
##========================================================================
## PHASE 4:
#    args = basic_args + gen_random
##========================================================================
## Comprehensive Test:
    args = basic_args + gen_test_routine + gen_random
##========================================================================


##========================================================================
## DO NOT MODIFY THIS PART
##========================================================================

    ## Pass help flag through
    try:
        if (sys.argv[1] == '-h'):
            args.append('-h')
    except:
        pass
    ## Call program
    aeadtvgen(args)
##========================================================================